@@ -4,12 +4,10 @@ from __future__ import division
 from __future__ import print_function
 
 import os
-import re
 import sys
 import types
 
 import pytest
-from _pytest.config import get_config
 from _pytest.config import PytestPluginManager
 from _pytest.main import EXIT_NOTESTSCOLLECTED
 from _pytest.main import Session
@@ -21,7 +19,7 @@ def pytestpm():
 
 
 class TestPytestPluginInteractions(object):
-    def test_addhooks_conftestplugin(self, testdir):
+    def test_addhooks_conftestplugin(self, testdir, _config_for_test):
         testdir.makepyfile(
             newhooks="""
             def pytest_myhook(xyz):
@@ -37,7 +35,7 @@ class TestPytestPluginInteractions(object):
                 return xyz + 1
         """
         )
-        config = get_config()
+        config = _config_for_test
         pm = config.pluginmanager
         pm.hook.pytest_addhooks.call_historic(
             kwargs=dict(pluginmanager=config.pluginmanager)
@@ -92,8 +90,8 @@ class TestPytestPluginInteractions(object):
         config.pluginmanager.register(A())
         assert len(values) == 2
 
-    def test_hook_tracing(self):
-        pytestpm = get_config().pluginmanager  # fully initialized with plugins
+    def test_hook_tracing(self, _config_for_test):
+        pytestpm = _config_for_test.pluginmanager  # fully initialized with plugins
         saveindent = []
 
         class api1(object):
@@ -165,10 +163,10 @@ def test_importplugin_error_message(testdir, pytestpm):
     with pytest.raises(ImportError) as excinfo:
         pytestpm.import_plugin("qwe")
 
-    expected_message = '.*Error importing plugin "qwe": Not possible to import: .'
-    expected_traceback = ".*in test_traceback"
-    assert re.match(expected_message, str(excinfo.value))
-    assert re.match(expected_traceback, str(excinfo.traceback[-1]))
+    assert str(excinfo.value).endswith(
+        'Error importing plugin "qwe": Not possible to import: ☺'
+    )
+    assert "in test_traceback" in str(excinfo.traceback[-1])
 
 
 class TestPytestPluginManager(object):
@@ -202,8 +200,8 @@ class TestPytestPluginManager(object):
         assert pytestpm.get_plugin("pytest_p1").__name__ == "pytest_p1"
         assert pytestpm.get_plugin("pytest_p2").__name__ == "pytest_p2"
 
-    def test_consider_module_import_module(self, testdir):
-        pytestpm = get_config().pluginmanager
+    def test_consider_module_import_module(self, testdir, _config_for_test):
+        pytestpm = _config_for_test.pluginmanager
         mod = types.ModuleType("x")
         mod.pytest_plugins = "pytest_a"
         aplugin = testdir.makepyfile(pytest_a="#")
@@ -313,6 +311,9 @@ class TestPytestPluginManagerBootstrapming(object):
         assert '"hello123"' in excinfo.value.args[0]
         pytestpm.consider_preparse(["-pno:hello123"])
 
+        # Handles -p without following arg (when used without argparse).
+        pytestpm.consider_preparse(["-p"])
+
     def test_plugin_prevent_register(self, pytestpm):
         pytestpm.consider_preparse(["xyz", "-p", "no:abc"])
         l1 = pytestpm.get_plugins()
@@ -346,3 +347,10 @@ class TestPytestPluginManagerBootstrapming(object):
         l2 = pytestpm.get_plugins()
         assert 42 not in l2
         assert 43 not in l2
+
+    def test_blocked_plugin_can_be_used(self, pytestpm):
+        pytestpm.consider_preparse(["xyz", "-p", "no:abc", "-p", "abc"])
+
+        assert pytestpm.has_plugin("abc")
+        assert not pytestpm.is_blocked("abc")
+        assert not pytestpm.is_blocked("pytest_abc")
